=head1 NAME

getObjectsForFuncGroup - Utility tool for CA SDM security auditing

Version 20160818

=head1 SYNOPSIS

pdm_perl getObjectsForFuncGroup.pl [options]

=head1 OPTIONS

=over 12

=item B<--help>

Prints the full help

=item B<--funcName>

Dumps a list of objects in alphabetical order that match the function group given as the argument

=item B<--debug>

Finishes processing after 10 objects, useful for quick test runs

=back

=head1 DESCRIPTION

This program will process the object names given by the local CA Service Desk Manager installation, process them one by one and in the end dump the objects into stdout. If the C<--funcName> is given then prints objects only for that function group. With no arguments dumps all objects for each function group.

While the output is printed into regular STDOUT, the status lines are printed into STDERR to avoid cluttering the output when redirecting the results into a file.

With C<--debug> flag set, will only process the first ten objects and then print the results as if that's all the objects there are.

=head1 LICENSE

Licensed under Creative Commons Attribution-ShareAlike 4.0 International Public License
L<https://creativecommons.org/licenses/by-sa/4.0/>

=head1 AUTHOR

Jussi Valkonen <L<jussi.valkonen@intronit.com>>

=cut

use warnings;
use strict;
use Getopt::Long;
use Pod::Usage;
use Data::Dumper;
$| = 1;

# Using Getopt::Long for param handling
Getopt::Long::Configure(qw(posix_default)); # Set long-params-only (no abbreviating --help to --h or -h)
my $funcName = '';
my $help;
my $debug;
if (!GetOptions(
	'funcName=s' => \$funcName, # If set, gets the options for the function group given, otherwise dumps options per each function group.
	'help|?' => \$help, # If set, prints the POD doc from the end and quits
	'debug' => \$debug, # If set, will only process 10 first objects and quit.
	)) {
	pod2usage(-verbose => 1, -noperldoc => 1); # Short help if the arguments are not recognized
	exit(0);
};

if ($help){
	pod2usage(-verbose => 2, -noperldoc => 1); # Full documentation, skip using perldoc and use Pod::Text instead
	exit(0);
}
$funcName = lc($funcName); # The function group name seems to be always all lower case


# Get the available objects
my $allObjects = parseObjnames();

# Process the objects in $allObjects and find out their function group, return a 
# hashref to hash of objects with $funcName as key and arrayref as value
my $objsForAllFuncGroups = getFGInfo($allObjects);

#print Dumper $objsForAllFuncGroups;

if ($funcName ne ""){ # Specific function group
	my $objsForFuncGroup = $objsForAllFuncGroups->{$funcName};

	# Sort the found objects names for nicer printing and post processing
	@$objsForFuncGroup = sort(@$objsForFuncGroup);

	print("Found ".scalar(@$objsForFuncGroup)." matching objects for function group '$funcName'\n");
	print(join("\n", @$objsForFuncGroup));

} else { # Dump mode, just dump all by func group
	# Get the hash keys (function group names) and sort them
	my @hashKeys = keys(%$objsForAllFuncGroups);
	@hashKeys = sort(@hashKeys);
	
	#print Dumper @hashKeys;
	#Loop through and print the value array in sorted order for each key
	for my $key (@hashKeys) {
		my @sorted = sort(@{$objsForAllFuncGroups->{$key}});
		print("Function group '$key'\n");
		print("\t".join("\n\t", @sorted)."\n");
		print("\n");
	}
}
exit(0);

#
# Parses the objnames output and returns the parsed results in a hashref 
# where the object name is the key and content is another hashref with table 
# as the only key.
#
sub parseObjnames {
	my @raw = `objnames`;
	chomp(@raw);
	
	my $ret = {};
	
	foreach my $line (@raw) {
		next if ($line =~ /^$/); # Empty line
		next if ($line =~ /^#.*/); # Comment line (begins with #)
		
		# Splitting line in two from space
		my @arr = split(/ /, $line);
		$ret->{$arr[0]} = { 'table' => $arr[1] };
	}
	return $ret;
}

#
# Process the list of objects given as arg0 and return a hashref of function
# names as keys and array of object names as values
#
sub getFGInfo {
	my $allObjects = shift; # arg0
	my $processed = {}; # Reference to an empty hash
	my ($total, $count) = (scalar(keys(%$allObjects)), 1);
	
	print(STDERR "Processing $total objects. Please be patient, this will take time.\n");
	print(STDERR "DEBUG flag set, will only process first 10 objects\n") if ($debug);
	
	foreach my $obj (keys(%$allObjects)) {
		my $captured = `bop_sinfo -f $obj | find "Func"`;
		chomp($captured);
		
		# Strip the space from both ends of the line
		$captured =~ s/^\s*|\s*$//g;
		
		# Split the line from the = sign and get the last trimmed part
		my @arr = split(/=/, $captured);
		next if (scalar(@arr) < 2); # No function group (array size less than 2), not interested

		# Clean up the line and store the result
		$arr[1] =~ s/^\s*|\s*$//g;
		if (!exists($processed->{$arr[1]})){ # Init the hash value if it didn't exist before
			$processed->{$arr[1]} = [];
		}
		push(@{$processed->{$arr[1]}}, $obj);

		# Status print. This loop will take a long time so better keep the 
		# user updated on the progress
		print(STDERR "\r\tProcessed $count/$total objects");

		last if ($debug && $count > 9);
		$count++;
	}
	print(STDERR "\n"); # Just slap a line feed after the fancy status output after the last round
	return $processed;
}
